@@ -16,7 +16,7 @@
 #' @param ci Logical scalar. Also return CI names?
 #' @param plain Logical scalar. Return the plain basenames only, ignoring
 #'   \code{subset} and \code{ci}?
-#' @param opm.fast Logical scalar. Produce the mapping gfor the
+#' @param opm.fast Logical scalar. Produce the mapping for the
 #'   \sQuote{opm-fast} method instead?
 #' @return Named list with old names as keys, new ones as values.
 #' @keywords internal
@@ -59,6 +59,33 @@ map_grofit_names <- function(subset = NULL, ci = TRUE, plain = FALSE,
 ################################################################################
 
 
+## NOTE: not an S4 method because manual dispatch
+
+
+#' Translate well coordinates
+#'
+#' Translate well coordinates to numeric indexes
+#'
+#' @param x Vector, formula or missing. Basically any \R object.
+#' @param names Character vector. Ignored unless \code{x} is a formula.
+#' @return Either \code{x}, \code{TRUE} or (if a formula) the result of
+#'   evaluating \code{x} in the context of \code{names}, converted to a mapping
+#'   from elements to indexes.
+#' @keywords internal
+#'
+well_index <- function(x, names) {
+  if (missing(x))
+    TRUE
+  else if (inherits(x, "formula"))
+    eval(x[[length(x)]], structure(as.list(seq_along(names)), .Names = names))
+  else
+    x
+}
+
+
+################################################################################
+
+
 ## NOTE: not an S4 method because conversion is done
 
 #' Map well names to substrates
@@ -106,25 +133,32 @@ map_well_names <- function(wells, plate, in.parens = FALSE, brackets = FALSE,
 #' Create a textual (listing-like) description.
 #'
 #' @param x Logical vector.
+#' @param html Logical scalar.
+#' @param ... Optional arguments passed to and from other methods.
 #' @return Character vector, one element per sentence.
 #' @keywords internal
 #'
-to_sentence <- function(x) UseMethod("to_sentence")
+to_sentence <- function(x, ...) UseMethod("to_sentence")
 
 #' @rdname to_sentence
 #' @method to_sentence logical
 #'
-to_sentence.logical <- function(x) {
+to_sentence.logical <- function(x, html, ...) {
   sentence <- function(x, what) {
-    if (length(x))
+    if (length(x)) {
+      if (html)
+        x <- substrate_info(x, "html")
       sprintf("%s for %s.", what, listing(x, style = "sentence"))
-    else
+    } else
       ""
   }
+  LL(html)
   isna <- is.na(x)
   n <- c("Positive", "Negative", "Ambiguous")
   result <- c(sentence(names(x)[x & !isna], n[1L]),
     sentence(names(x)[!x & !isna], n[2L]), sentence(names(x)[isna], n[3L]))
+  if (html)
+    result <- sprintf("<div>%s</div>", result)
   names(result) <- n
   result
 }
@@ -137,7 +171,7 @@ to_sentence.logical <- function(x) {
 #'
 #' This creates a textual listing of the discretized values. This is useful to
 #' describe OmniLog\eqn{\textsuperscript{\textregistered}}{(R)} phenotype
-#' microarray results in a manuscript.
+#' microarray results in a scientific manuscript.
 #'
 #' @param x \code{\link{OPMD}} or \code{\link{OPMS}} object.
 #' @param downcase Logical scalar passed to \code{\link{wells}}.
@@ -153,9 +187,13 @@ to_sentence.logical <- function(x) {
 #'   (if any).
 #' @param exact Logical scalar passed to \code{\link{metadata}}.
 #' @param strict Logical scalar also passed to \code{\link{metadata}}.
+#' @param html Logical scalar. Convert to \acronym{HTML}? This involves Greek
+#'   letters and paragraph (\sQuote{div}) tags.
 #' @param ... Optional arguments passed between the methods or to
 #'   \code{\link{wells}}.
-#' @return Character vector or matrix. See the examples for details.
+#' @return Character vector or matrix with additional class atribute
+#'   \sQuote{OPMD_Listing} or \sQuote{OPMS_Listing}. See the examples for
+#'   details.
 #' @export
 #' @family naming-functions
 #' @keywords character category
@@ -167,53 +205,70 @@ to_sentence.logical <- function(x) {
 #'
 #' # this yields one sentence for each kind of reaction:
 #' (x <- listing(vaas_1))
-#' stopifnot(is.character(x), length(x) == 3, !is.null(names(x)))
+#' stopifnot(inherits(x, "OPMD_Listing"), is.character(x), length(x) == 3,
+#'   !is.null(names(x)))
+#'
+#' # including HTML tags
+#' (y <- listing(vaas_1, html = TRUE))
+#' stopifnot(inherits(y, "OPMD_Listing"), is.character(x), nchar(y) > nchar(x),
+#'   !is.null(names(x)))
 #'
 #' # 'OPMS' method
 #' data(vaas_4)
 #'
-#' # no grouping, no names
-#' (y <- listing(vaas_4, as.groups = NULL))
-#' stopifnot(is.matrix(y), dim(y) == c(4, 3))
-#' stopifnot(is.null(rownames(y)), !is.null(colnames(y)))
+#' # no grouping, no names (numbering used instead for row names)
+#' (y <- listing(vaas_4[1:2], as.groups = NULL))
+#' stopifnot(inherits(y, "OPMS_Listing"), is.matrix(y), dim(y) == c(2, 3))
+#' stopifnot(!is.null(rownames(y)), !is.null(colnames(y)))
 #'
 #' # in effect no grouping, but names
-#' (y <- listing(vaas_4, as.groups = list("Species", "Strain")))
-#' stopifnot(is.matrix(y), dim(y) == c(4, 3))
+#' (y <- listing(vaas_4[1:2], as.groups = list("Species", "Strain")))
+#' stopifnot(inherits(y, "OPMS_Listing"), is.matrix(y), dim(y) == c(2, 3))
 #' stopifnot(!is.null(rownames(y)), !is.null(colnames(y)))
 #'
 #' # two groups
 #' (y <- listing(vaas_4, as.groups = list("Species")))
-#' stopifnot(is.matrix(y), dim(y) == c(2, 3))
+#' stopifnot(inherits(y, "OPMS_Listing"), is.matrix(y), dim(y) == c(2, 3))
 #' stopifnot(!is.null(rownames(y)), !is.null(colnames(y)))
 #'
 setGeneric("listing")
 
 setMethod("listing", OPMD, function(x, downcase = TRUE, full = TRUE,
-    in.parens = FALSE, ...) {
+    in.parens = FALSE, html = FALSE, ...) {
   res <- discretized(x)
   names(res) <- wells(object = x, full = full, in.parens = in.parens,
     downcase = downcase, ...)
-  to_sentence(res)
+  res <- to_sentence(res, html)
+  class(res) <- "OPMD_Listing"
+  attr(res, "html") <- html
+  res
 }, sealed = SEALED)
 
 setMethod("listing", OPMS, function(x, as.groups, cutoff = 0.5, sep = " ",
     exact = TRUE, strict = TRUE, downcase = TRUE, full = TRUE,
-    in.parens = FALSE, ...) {
+    in.parens = FALSE, html = FALSE, ...) {
+  add_stuff <- function(x, html) {
+    class(x) <- "OPMS_Listing"
+    attr(x, "html") <- html
+    x
+  }
   LL(cutoff, sep)
-  if (!length(as.groups))
-    return(do.call(rbind, lapply(X = x@plates, FUN = listing,
-      downcase = downcase, full = full, in.parens = in.parens, ...)))
+  if (!length(as.groups)) {
+    res <- do.call(rbind, lapply(X = x@plates, FUN = listing, html = html,
+      downcase = downcase, full = full, in.parens = in.parens, ...))
+    rownames(res) <- seq.int(nrow(res))
+    return(add_stuff(res, html))
+  }
   disc <- extract(object = x, subset = "disc", as.groups = as.groups,
     sep = sep, exact = exact, strict = strict, downcase = downcase,
     full = full, in.parens = in.parens, dataframe = FALSE, as.labels = NULL,
     ...)
   groups <- attr(disc, "row.groups")
-  t(vapply(levels(groups), function(group) {
-    y <- disc[groups == group, , drop = FALSE]
-    y <- apply(y, 2L, reduce_to_mode, cutoff = cutoff, use.na = TRUE)
-    to_sentence(y)
-  }, character(3L)))
+  add_stuff(t(vapply(levels(groups), function(group) {
+      y <- disc[groups == group, , drop = FALSE]
+      y <- apply(y, 2L, reduce_to_mode, cutoff = cutoff, use.na = TRUE)
+      to_sentence(y, html)
+    }, character(3L))), html)
 }, sealed = SEALED)
 
 
@@ -231,7 +286,8 @@ setMethod("listing", OPMS, function(x, as.groups, cutoff = 0.5, sep = " ",
 #'   \code{\link{plate_type}}. \code{\link{plate_type}} is applied before
 #'   searching for the substrate names, and partial matching is allowed.
 #' @param well Character vector of original well names (coordinates on the
-#'   plate), or integer vector, or convertible to such.
+#'   plate), or integer vector, or convertible to such, or formula. The formula
+#'   allows for sequences of well coordinates; see the examples for details.
 #' @export
 #' @return Character vector or matrix (depending on the length of \code{plate}),
 #'   containing \code{NA} values for plates and wells that could not be
@@ -242,13 +298,20 @@ setMethod("listing", OPMS, function(x, as.groups, cutoff = 0.5, sep = " ",
 #' x <- c("A01", "B10")
 #' (y <- well_to_substrate("PM1", x))
 #' stopifnot(nchar(y) > nchar(x))
+#' # formula yields same result
+#' stopifnot(identical(y, well_to_substrate("PM1", ~ c(A01, B10))))
+#' # using a sequence of well coordinates
+#' stopifnot(nrow(well_to_substrate("PM1", ~ C02:C06)) == 5) # well sequence
+#' stopifnot(nrow(well_to_substrate("PM1")) == 96) # all wells by default
 #'
-well_to_substrate <- function(plate, well = 1L:96L) {
+well_to_substrate <- function(plate, well = TRUE) {
+  well <- well_index(well, rownames(WELL_MAP))
   pos <- pmatch(plate_type(plate), colnames(WELL_MAP))
   found <- !is.na(pos)
   result <- WELL_MAP[well, pos[found], drop = FALSE]
-  others <- matrix(data = NA_character_, nrow = length(well),
-    ncol = length(plate[!found]), dimnames = list(well, plate[!found]))
+  others <- matrix(data = NA_character_, nrow = nrow(result),
+    ncol = length(plate[!found]),
+    dimnames = list(rownames(result), plate[!found]))
   cbind(result, others)[]
 }
 
@@ -263,15 +326,18 @@ well_to_substrate <- function(plate, well = 1L:96L) {
 #' regular-expression matching.
 #'
 #' @param object Query character vector or factor.
-#' @param search Character scalar indicating the search mode. If \sQuote{exact},
-#'   query names must exactly match (parts of) the well annotations. If
-#'   \sQuote{glob}, shell globbing is used. If \sQuote{approx}, approximate
-#'   matching is used; the number or proportion of errors allowed is set using
-#'   \code{max.dev}, and neither globbing or regular-expression matching is done
-#'   in that case. If \sQuote{regex}, regular-expression matching is used. If
-#'   \sQuote{pmatch}, uses \code{pmatch} from the \pkg{base} package. All
-#'   matching is case-insensitive except for \sQuote{exact} and \sQuote{pmatch}
-#'   search modes.
+#' @param search Character scalar indicating the search mode. \describe{
+#'   \item{exact}{Query names must exactly match (parts of) the well
+#'   annotations.}
+#'   \item{glob}{Shell globbing is used.}
+#'   \item{approx}{Approximate matching is used; the number or proportion of
+#'   errors allowed is set using \code{max.dev}, and neither globbing or
+#'   regular-expression matching is done in that case.}
+#'   \item{regex}{Regular-expression matching is used.}
+#'   \item{pmatch}{Uses \code{pmatch} from the \pkg{base} package.}
+#'   }
+#'   All matching is case-insensitive except for \sQuote{exact} and
+#'   \sQuote{pmatch} search modes.
 #' @param max.dev Numeric scalar indicating the maximum allowed deviation. If <
 #'   1, the proportion of characters that might deviate, otherwise their
 #'   absolute number. It can also be a list; see the \sQuote{max.distance}
@@ -343,9 +409,7 @@ setMethod("find_substrate", "character", function(object,
 #' Identify positions of substrates
 #'
 #' Identify the positions of substrates, i.e. the plate(s) and well(s) in which
-#' they occur. The query names must be written exactly as used in the stored
-#' plate annotations. To determine their spelling, use
-#' \code{\link{find_substrate}}.
+#' they occur.
 #'
 #' @param object Query character vector or query list.
 #' @param ... Optional arguments passed between the methods.
@@ -354,6 +418,8 @@ setMethod("find_substrate", "character", function(object,
 #'   nothing was found), with one row per position found, the plate name in the
 #'   first column and the well name in the second. The names of this list
 #'   correspond to \code{names}. The list method returns lists of such lists.
+#' @details  The query names must be written exactly as used in the stored plate
+#'   annotations. To determine their spelling, use \code{\link{find_substrate}}.
 #' @family naming-functions
 #' @keywords utilities
 #' @examples
@@ -367,7 +433,9 @@ setMethod("find_substrate", "character", function(object,
 #' stopifnot(identical(y, x))
 #'
 #' # List method
-#' (x <- find_positions(find_substrate(c("D-Glucose", "D-Gloucose"))))
+#' x <- find_positions(find_substrate(c("D-Glucose", "D-Gloucose")))
+#' x[[1]][1:3]
+#' x[[2]]
 #' stopifnot(length(x[[1]]) > length(x[[2]]))
 #'
 setGeneric("find_positions",
@@ -378,7 +446,7 @@ setMethod("find_positions", "character", function(object) {
   sapply(object, FUN = function(name) {
     result <- which(WELL_MAP == name, arr.ind = TRUE)
     matrix(c(plates[result[, 2L]], rownames(result)), ncol = 2L,
-      dimnames = list(NULL, c("Plate", "Well")))
+      dimnames = list(NULL, RESERVED_NAMES[c("plate", "well")]))
   }, simplify = FALSE)
 }, sealed = SEALED)
 
@@ -394,17 +462,15 @@ setMethod("find_positions", "list", function(object) {
 #' Provide information on substrates
 #'
 #' Return information on substrates such as their \acronym{CAS} number or
-#' \acronym{KEGG} ID. The query names must be written exactly as used in the
-#' stored plate annotations. To determine their spelling, use
-#' \code{\link{find_substrate}}. Alternatively, this functions converts the
-#' substrate names to lower case, protecting one-letter specifiers, acronyms and
-#' chemical symbols, and translating relevant characters from the Greek
-#' alphabet.
+#' \acronym{KEGG} ID. Alternatively, convert substrate names to lower case,
+#' protecting one-letter specifiers, acronyms and chemical symbols, and
+#' translating relevant characters from the Greek alphabet, or only translate
+#' to Greek letters, optionally formatted with \acronym{HTML} tags.
 #'
 #' @param object Query character vector or query list.
 #' @param what Character scalar indicating which kind of information to output.
 #'   See the references for the background of each possible value.
-#'   \sQuote{downcase} is special; see above.
+#'   \sQuote{downcase}, \sQuote{greek} and \sQuote{html} are special; see above.
 #' @param ... Optional arguments passed between the methods.
 #' @export
 #' @return The character method returns a character vector with \code{object}
@@ -418,8 +484,11 @@ setMethod("find_positions", "list", function(object) {
 #' @references \url{http://www.genome.jp/kegg/}
 #' @references \url{http://metacyc.org/}
 #' @references \url{http://www.ncbi.nlm.nih.gov/mesh}
-#' @note Currently the information is incomplete, particularly for the PM-M
-#'   plates. While it should eventually be possible to link all substrates to
+#' @details The query names must be written exactly as used in the stored plate
+#'   annotations. To determine their spelling, use \code{\link{find_substrate}}.
+#'
+#'   Currently the information is incomplete, particularly for the PM-M plates.
+#'   While it should eventually be possible to link all substrates to
 #'   \acronym{CAS} numbers, they are not necessarily contained in the other
 #'   databases.
 #' @examples
@@ -445,7 +514,20 @@ setGeneric("substrate_info",
   function(object, ...) standardGeneric("substrate_info"))
 
 setMethod("substrate_info", "character", function(object,
-    what = c("cas", "kegg", "metacyc", "mesh", "downcase")) {
+    what = c("cas", "kegg", "metacyc", "mesh", "downcase", "greek", "html")) {
+  map_words <- function(x, fun, ...) {
+    y <- strsplit(x, "\\w+", perl = TRUE)
+    x <- strsplit(x, "\\W+", perl = TRUE)
+    bad <- !vapply(x, function(value) nzchar(value[1L]), logical(1L))
+    x[bad] <- lapply(x[bad], `[`, i = -1L)
+    bad <- vapply(x, length, integer(1L)) < vapply(y, length, integer(1L))
+    x[bad] <- lapply(x[bad], function(value) c(value, ""))
+    x <- lapply(X = x, FUN = fun, ...) # fun() must keep the length!
+    mapply(paste, y, x, MoreArgs = list(sep = "", collapse = ""))
+  }
+  convert_greek <- function(x, how) {
+    map_words(x, fun = map_values, mapping = GREEK_LETTERS[, how])
+  }
   safe_downcase <- function(x) {
     good_case <- function(x) {
       bad <- nchar(x) > 1L # avoid changing acronyms and chemical elements
@@ -454,20 +536,17 @@ setMethod("substrate_info", "character", function(object,
       x[bad] <- tolower(x[bad])
       x
     }
-    y <- strsplit(x, "\\w+", perl = TRUE)
-    x <- strsplit(x, "\\W+", perl = TRUE)
-    bad <- !vapply(x, function(value) nzchar(value[1L]), logical(1L))
-    x[bad] <- lapply(x[bad], `[`, i = -1L)
-    bad <- vapply(x, length, integer(1L)) < vapply(y, length, integer(1L))
-    x[bad] <- lapply(x[bad], function(value) c(value, ""))
-    x <- lapply(x, good_case)
-    x <- map_values(x, GREEK_LETTERS[, "plain"])
-    mapply(paste, y, x, MoreArgs = list(sep = "", collapse = ""))
+    map_words(x, function(y) map_values(good_case(y), GREEK_LETTERS[, "plain"]))
   }
-  structure(.Data = case(what <- match.arg(what),
-    downcase = safe_downcase(object), kegg =, metacyc =, mesh =,
+  result <- case(what <- match.arg(what),
+    downcase = safe_downcase(object),
+    greek = convert_greek(object, "plain"),
+    html = convert_greek(object, "html"),
+    kegg =, metacyc =, mesh =,
     cas = SUBSTRATE_INFO[match(object, rownames(SUBSTRATE_INFO)), toupper(what)]
-  ), .Names = object)
+  )
+  names(result) <- object
+  result
 }, sealed = SEALED)
 
 setMethod("substrate_info", "list", function(object, ...) {
