@@ -5,7 +5,7 @@
 #
 # run_opm.R -- R script for non-interactive use of the opm package
 #
-# (C) 2012 by Markus Goeker (markus [DOT] goeker [AT] dsmz [DOT] de)
+# (C) 2013 by Markus Goeker (markus [DOT] goeker [AT] dsmz [DOT] de)
 #
 # This script is distributed under the terms of the GPL. For further details
 # see the opm package.
@@ -20,17 +20,77 @@ invisible(lapply(c("optparse", "pkgutils", "opm"), library, quietly = TRUE,
 MD.OUTFILE <- "metadata.csv"
 RESULT <- c(
   "Clean filenames by removing non-word characters except dots and dashes.",
-  "Draw plots as postscript files, one per input file.",
+  "Draw level plots into graphics files, one per input file.",
   "Split OmniLog(R) CSV files into one file per plate.",
   "Collect a template for adding metadata.",
-  "Convert input OmniLog(R) CSV (or opm YAML) files to opm YAML."
+  "Draw xy plots into graphics files, one per input file.",
+  "Convert input OmniLog(R) CSV (or opm YAML or JSON) files to opm YAML.",
+  "Convert input OmniLog(R) CSV (or opm YAML or JSON) files to opm JSON."
 )
-names(RESULT) <- c("clean", "plot", "split", "template", "yaml")
+names(RESULT) <- c("clean", "levelplot", "split", "template", "xyplot",
+  "yaml", "json")
+AGGREGATION <- c(
+  "No estimation of curve parameters.",
+  "Fast estimation (only two parameters).",
+  "Approach from grofit package.",
+  "Using p-splines.",
+  "Using smoothing splines.",
+  "Using thin-plate splines"
+)
+names(AGGREGATION) <- c("no", "fast", "grofit", "p", "smooth", "thin")
 
 
 ################################################################################
 #
-# Functions for each running mode
+# Helper functions
+#
+
+
+make_md_args <- function(opt) {
+  if (is.null(opt$mdfile))
+    NULL
+  else
+    list(md = opt$mdfile, sep = opt$sep, replace = opt$exchange)
+}
+
+
+#-------------------------------------------------------------------------------
+
+
+make_disc_args <- function(opt) {
+  if (opt$discretize)
+    list(cutoff = opt$weak, plain = FALSE)
+  else
+    NULL
+}
+
+
+#-------------------------------------------------------------------------------
+
+
+make_aggr_args <- function(opt) {
+  aggr_args <- function(opt, method, spline) {
+    x <- list(boot = opt$bootstrap, verbose = !opt$quiet,
+      cores = opt$processes)
+    x$method <- method
+    if (length(spline))
+      x$options <- set_spline_options(type = spline)
+    x
+  }
+  case(match.arg(opt$aggregate, names(AGGREGATION)),
+    no = NULL,
+    fast = aggr_args(opt, "opm-fast", NULL),
+    grofit = aggr_args(opt, "grofit", NULL),
+    p = aggr_args(opt, "splines", "p.spline"),
+    smooth = aggr_args(opt, "splines", "smooth.spline"),
+    thin = aggr_args(opt, "splines", "tp.spline")
+  )
+}
+
+
+################################################################################
+#
+# Functions for each output mode
 #
 
 
@@ -44,21 +104,11 @@ run_clean_mode <- function(input, opt) {
 
 
 run_plot_mode <- function(input, opt) {
-  if (!nzchar(opt$dir))
-    opt$dir <- NULL
-  plot_fun <- if (opt$level)
-    level_plot
-  else
-    xy_plot
-  io_fun <- function(infile, outfile) {
-    x <- read_opm(infile, gen.iii = opt$type)
-    postscript(outfile)
-    print(plot_fun(x))
-    dev.off()
-  }
-  batch_process(names = input, proc = opt$processes, out.ext = "ps",
-    io.fun = io_fun, outdir = opt$dir, verbose = !opt$quiet,
-    overwrite = opt$overwrite, include = opt$include, exclude = opt$exclude)
+  batch_opm(names = input, proc = opt$processes, disc.args = NULL,
+    aggr.args = NULL, md.args = make_md_args(opt), outdir = opt$dir,
+    verbose = !opt$quiet, overwrite = opt$overwrite, include = opt$include,
+    exclude = opt$exclude, gen.iii = opt$type, device = opt$format,
+    output = opt$result)
 }
 
 
@@ -95,32 +145,16 @@ run_template_mode <- function(input, opt) {
 
 
 run_yaml_mode <- function(input, opt) {
-  if (opt$coarse || opt$fast) {
-    proc <- opt$processes
+  if (opt$coarse || opt$aggregate == "fast") {
+    proc <- opt$processes # this must be run before make_aggr_args()
     opt$processes <- 1L
   } else
     proc <- 1L
-  if (!nzchar(opt$dir))
-    opt$dir <- NULL
-  if (opt$aggregate) {
-    aggr.args <- list(boot = opt$bootstrap, verbose = !opt$quiet,
-      cores = opt$processes)
-    if (opt$fast)
-      aggr.args$method <- "opm-fast"
-  } else
-    aggr.args <- NULL
-  if (opt$discretize)
-    disc.args <- list(cutoff = opt$weak, plain = FALSE)
-  else
-    disc.args <- NULL
-  md.args <- if (is.null(opt$mdfile))
-    NULL
-  else
-    list(md = opt$mdfile, sep = opt$sep, replace = opt$exchange)
-  batch_opm_to_yaml(names = input, proc = proc, disc.args = disc.args,
-    aggr.args = aggr.args, md.args = md.args, outdir = opt$dir,
-    verbose = !opt$quiet, overwrite = opt$overwrite, include = opt$include,
-    exclude = opt$exclude, gen.iii = opt$type)
+  batch_opm(names = input, proc = proc, disc.args = make_disc_args(opt),
+    outdir = opt$dir, aggr.args = make_aggr_args(opt),
+    md.args = make_md_args(opt), verbose = !opt$quiet,
+    overwrite = opt$overwrite, include = opt$include,
+    exclude = opt$exclude, gen.iii = opt$type, output = opt$result)
 }
 
 
@@ -132,11 +166,12 @@ run_yaml_mode <- function(input, opt) {
 
 option.parser <- OptionParser(option_list = list(
 
-  make_option(c("-a", "--aggregate"), action = "store_true", default = FALSE,
-    help = "Aggregate by estimating curve parameters [default: %default]"),
+  make_option(c("-a", "--aggregate"), type = "character", default = "no",
+    help = "Aggregate by estimating curve parameters [default: %default]",
+    metavar = "METHOD"),
 
   make_option(c("-b", "--bootstrap"), type = "integer", default = 100L,
-    help = "Number of bootstrap replicates when aggreating [default: %default]",
+    help = "# bootstrap replicates when aggregating [default: %default]",
     metavar = "NUMBER"),
 
   make_option(c("-c", "--coarse"), action = "store_true", default = FALSE,
@@ -149,13 +184,12 @@ option.parser <- OptionParser(option_list = list(
     help = "File exclusion globbing pattern [default: <none>]",
     metavar = "PATTERN"),
 
-  make_option(c("-f", "--fast"), action = "store_true", default = FALSE,
-    help = "When aggregating, use fast method [default: %default]"),
+  make_option(c("-f", "--format"), type = "character", default = "postscript",
+    help = "Graphics output format [default: %default]", metavar = "NAME"),
 
-  # A bug in Rscript causes '-g' to generate strange warning messages.
-  # See https://stat.ethz.ch/pipermail/r-devel/2008-January/047944.html
-  #make_option(c("-G", "--Gen3"), action = "store_true", default = FALSE,
-  #  help = "Change plate type to generation III [default: %default]"),
+  ## A bug in Rscript causes '-g' to generate strange warning messages.
+  ## See https://stat.ethz.ch/pipermail/r-devel/2008-January/047944.html
+  # g
 
   # h
 
@@ -163,10 +197,7 @@ option.parser <- OptionParser(option_list = list(
     help = "File inclusion globbing pattern [default: <see package>]",
     metavar = "PATTERN"),
 
-  # j, k
-
-  make_option(c("-l", "--level"), action = "store_true", default = FALSE,
-    help = "When plotting, draw levelplot [default: %default]"),
+  # j, k, l
 
   make_option(c("-m", "--mdfile"), type = "character",
     default = NULL, metavar = "NAME",
@@ -181,7 +212,7 @@ option.parser <- OptionParser(option_list = list(
     metavar = "MODE"),
 
   make_option(c("-p", "--processes"), type = "integer", default = 1L,
-    help = paste("Number of processes to spawn (>1 needs 'multicore')",
+    help = paste("Number of processes to spawn (>1 impossible under Windows)",
       "[default: %default]"),
     metavar = "NUMBER"),
 
@@ -216,7 +247,7 @@ option.parser <- OptionParser(option_list = list(
   make_option(c("-z", "--discretize"), action = "store_true", default = FALSE,
     help = "Discretize after estimating curve parameters [default: %default]")
 
-))
+), usage = "%prog [options] [directories/files]")
 
 
 opt <- parse_args(option.parser, positional_arguments = TRUE)
@@ -224,6 +255,8 @@ input <- opt$args
 opt <- opt$options
 if (is.null(opt$include))
   opt$include <- list()
+if (!nzchar(opt$dir))
+  opt$dir <- NULL
 
 
 ################################################################################
@@ -232,10 +265,12 @@ if (is.null(opt$include))
 #
 
 
-if (length(input) == 0L) {
+if (!length(input)) {
   print_help(option.parser)
-  message(listing(RESULT, header = "The output modes are:", footer = "",
-    prepend = 5L, indent = 10L))
+  cat(listing(RESULT, header = "The output modes are:", footer = "",
+    prepend = 5L, indent = 10L), sep = "\n")
+  cat(listing(AGGREGATION, header = "The aggregation modes are:", footer = "",
+    prepend = 5L, indent = 10L), sep = "\n")
   quit(status = 1L)
 }
 
@@ -245,9 +280,11 @@ invisible(opm_opt(file.encoding = opt$encoding))
 
 case(match.arg(opt$result, names(RESULT)),
   clean = run_clean_mode(input, opt),
-  plot = run_plot_mode(input, opt),
+  xyplot =,
+  levelplot = run_plot_mode(input, opt),
   split = run_split_mode(input, opt),
   template = run_template_mode(input, opt),
+  json =,
   yaml = run_yaml_mode(input, opt)
 )
 
