@@ -8,6 +8,39 @@ if (!exists("TEST.DIR"))
   attach(objects_for_testing())
 
 
+# Test whether or not object 'x' could be submitted to ci_plot.
+#
+is_ci_plottable <- function(x) {
+  param_column_ok <- function(x) {
+    pat <- sprintf("^(%s)", paste(param_names(), collapse = "|"))
+    x <- sub(pat, "", x, perl = TRUE)
+    y <- c("", " CI95 low", " CI95 high")
+    !(length(x) %% length(y)) && all(x == y)
+  }
+  data_columns_ok <- function(x) {
+    rest <- seq.int(nrow(x)) %% 3L
+    all(x[rest == 1L, , drop = FALSE] >= x[rest == 2L, , drop = FALSE]) &&
+      all(x[rest == 1L, , drop = FALSE] <= x[rest == 0L, , drop = FALSE])
+  }
+  is.data.frame(x) &&
+    !is.na(pos <- match(RESERVED_NAMES[["parameter"]], colnames(x))) &&
+    param_column_ok(x[, pos]) &&
+    data_columns_ok(x[, (pos + 1L):ncol(x), drop = FALSE])
+}
+
+
+A.VALUES <- extract(c(THIN.AGG, THIN.AGG, THIN.AGG),
+  as.labels = list("organism", "run"),
+  subset = "A", dataframe = TRUE)
+
+NESTED.MD <- SMALL.WITH.MD
+metadata(NESTED.MD) <- list(A = list(B = 63, C = 64), K = "N")
+tmp <- SMALL.WITH.MD
+metadata(tmp) <- list(A = list(F = 63, C = 64), K = "T", O = letters)
+NESTED.MD <- c(NESTED.MD, tmp)
+rm(tmp)
+
+
 ################################################################################
 
 
@@ -123,7 +156,7 @@ test_that("the plate type of OPMS objects can be changed", {
 
 ## flatten
 test_that("example data can be flattened", {
-  base.colnames <- c("Time", "Well", "Value")
+  base.colnames <- unname(RESERVED_NAMES[c("time", "well", "value")])
   flat <- flatten(SMALL)
   expect_is(flat, "data.frame")
   expect_equal(colnames(flat), base.colnames)
@@ -135,7 +168,7 @@ test_that("example data can be flattened", {
 
 ## flatten
 test_that("example data with metadata can be flattened", {
-  base.colnames <- c("Time", "Well", "Value")
+  base.colnames <- unname(RESERVED_NAMES[c("time", "well", "value")])
   flat <- flatten(SMALL.WITH.MD, include = list("Organism"))
   expect_is(flat, "data.frame")
   expect_equal(colnames(flat), c("Organism", base.colnames))
@@ -147,7 +180,7 @@ test_that("example data with metadata can be flattened", {
 
 ## flatten
 test_that("example data can be flattened with fixed entries", {
-  base.colnames <- c("Time", "Well", "Value")
+  base.colnames <- unname(RESERVED_NAMES[c("time", "well", "value")])
   flat <- flatten(SMALL.WITH.MD, fixed = list(A = 33, B = "zzz"))
   expect_is(flat, "data.frame")
   expect_equal(colnames(flat), c("A", "B", base.colnames))
@@ -162,7 +195,7 @@ test_that("example data can be flattened with fixed entries", {
 ## flatten
 test_that("OPMS objects can be flattened", {
   opms.input <- OPMS.INPUT[, 1L:10L]
-  base.colnames <- c("Plate", "Time", "Well", "Value")
+  base.colnames <- unname(RESERVED_NAMES[c("plate", "time", "well", "value")])
   flat <- flatten(opms.input)
   expect_is(flat, "data.frame")
   expect_equal(colnames(flat), base.colnames)
@@ -175,7 +208,7 @@ test_that("OPMS objects can be flattened", {
 ## flatten
 test_that("OPMS objects including metadata can be flattened", {
   opms.input <- OPMS.INPUT[, 1L:10L]
-  base.colnames <- c("Plate", "Time", "Well", "Value")
+  base.colnames <- unname(RESERVED_NAMES[c("plate", "time", "well", "value")])
   # Flatten with metadata no. 1
   flat <- flatten(opms.input, include = list("organism"))
   expect_is(flat, "data.frame")
@@ -199,7 +232,76 @@ test_that("OPMS objects including metadata can be flattened", {
 
 
 ## extract_columns
-## UNTESTED
+test_that("extract_columns() gets nested metadata right", {
+
+  # 1
+  got <- extract_columns(NESTED.MD, list(c("A", "C"), "K"))
+  expect_equal(names(got), c("A.C", "K"))
+
+  # 2
+  got <- extract_columns(NESTED.MD, list(u = c("A", "C"), v = "K"))
+  expect_equal(names(got), c("u", "v"))
+
+  # 3
+  old <- opm_opt(key.join = "~")
+  on.exit(opm_opt(old))
+  got <- extract_columns(NESTED.MD, list(c("A", "C"), "K"))
+  expect_equal(names(got), c("A~C", "K"))
+  opm_opt(old)
+
+  # 4
+  got <- extract_columns(NESTED.MD, list(u = "A"))
+  expect_equal(names(got), c("u.B", "u.C"))
+
+  # 4
+  got <- extract_columns(NESTED.MD, list(z = "K"))
+  expect_equal(names(got), "z")
+
+})
+
+
+## extract_columns
+test_that("extract_columns() gets metadata right using character vector", {
+
+  # 1
+  got <- extract_columns(NESTED.MD, "A")
+  expect_equal(names(got), c("B", "C"))
+
+  # 2
+  got <- extract_columns(NESTED.MD, c("A", "C"))
+  expect_equal(names(got), "A.C")
+
+  # 3
+  old <- opm_opt(key.join = "~")
+  on.exit(opm_opt(old))
+  got <- extract_columns(NESTED.MD, c("A", "C"))
+  expect_equal(names(got), "A~C")
+
+})
+
+
+## extract_columns
+test_that("extract_columns() gets metadata right if a formula is used", {
+
+  # 1
+  got <- extract_columns(NESTED.MD, ~ A$C + K)
+  expect_equal(names(got), c("A.C", "K"))
+
+  # 2
+  got <- extract_columns(NESTED.MD, ~ A$C)
+  expect_equal(names(got), "A.C")
+
+  # 3
+  got <- extract_columns(NESTED.MD, ~ K)
+  expect_equal(names(got), "K")
+
+  # 4
+  old <- opm_opt(key.join = "~")
+  on.exit(opm_opt(old))
+  got <- extract_columns(NESTED.MD, ~ A$C + K)
+  expect_equal(names(got), c("A~C", "K"))
+
+})
 
 
 ## sort
@@ -286,7 +388,8 @@ test_that("aggregated parameters can be extracted as dataframe", {
     subset = "lambda", dataframe = TRUE, sep = "***")
   expect_is(mat, "data.frame")
   expect_equal(dim(mat), c(2L, 99L))
-  expect_equal(colnames(mat), c("organism", "run", "Parameter",
+  expect_equal(colnames(mat), c("organism", "run",
+    RESERVED_NAMES[["parameter"]],
     wells(THIN.AGG, full = TRUE)))
   expect_true(all(sapply(mat[, 1L:3L], is.factor)))
   expect_true(all(sapply(mat[, 4L:99L], is.numeric)))
@@ -299,7 +402,8 @@ test_that("aggregated parameters can be extracted as dataframe", {
     as.groups = list("run", "organism"))
   expect_is(mat, "data.frame")
   expect_equal(dim(mat), c(2L, 101L))
-  expect_equal(colnames(mat), c("organism", "run", "Parameter",
+  expect_equal(colnames(mat), c("organism", "run",
+    RESERVED_NAMES[["parameter"]],
     wells(THIN.AGG, full = TRUE), "run", "organism"))
   expect_true(all(sapply(mat[, 1L:3L], is.factor)))
   expect_true(all(sapply(mat[, 4L:99L], is.numeric)))
@@ -320,7 +424,8 @@ test_that("aggregated parameters can be extracted as dataframe with CIs", {
     subset = "lambda", dataframe = TRUE, sep = "***", ci = TRUE)
   expect_is(mat, "data.frame")
   expect_equal(dim(mat), c(6L, 99L))
-  expect_equal(colnames(mat), c("organism", "run", "Parameter",
+  expect_equal(colnames(mat), c("organism", "run",
+    RESERVED_NAMES[["parameter"]],
     wells(THIN.AGG, full = TRUE)))
   expect_true(all(sapply(mat[, 1L:3L], is.factor)))
   expect_true(all(sapply(mat[, 4L:99L], is.numeric)))
@@ -335,7 +440,8 @@ test_that("aggregated parameters can be extracted as dataframe with CIs", {
     as.groups = list("run", "organism"))
   expect_is(mat, "data.frame")
   expect_equal(dim(mat), c(6L, 101L))
-  expect_equal(colnames(mat), c("organism", "run", "Parameter",
+  expect_equal(colnames(mat), c("organism", "run",
+    RESERVED_NAMES[["parameter"]],
     wells(THIN.AGG, full = TRUE), "run", "organism"))
   expect_true(all(sapply(mat[, 1L:3L], is.factor)))
   expect_true(all(sapply(mat[, 4L:99L], is.numeric)))
@@ -351,6 +457,115 @@ test_that("aggregated parameters can be extracted as dataframe with CIs", {
 })
 
 
+## extract
+test_that("extract works without grouping and without normalisation", {
+  x <- extract(object = A.VALUES, as.groups = FALSE, norm.per = "none")
+  expect_is(x, "data.frame")
+  expect_false(is_ci_plottable(x))
+  expect_equal(x, A.VALUES)
+  expect_error(ci_plot(x)) # no CI were computed
+})
+
+
+## extract
+test_that("extract works with grouping and without normalisation", {
+  x <- extract(object = A.VALUES, as.groups = TRUE, norm.per = "none")
+  expect_is(x, "data.frame")
+  expect_equal(dim(x), c(6L, 99L))
+  expect_true(is_ci_plottable(x))
+  got <- ci_plot(x[, 1L:9L])
+  expect_equal(got, c("1: Bacillus simplex 3", "2: Bacillus simplex 4"))
+})
+
+
+## extract
+test_that("extract works with grouping and 'plate.sub' normalisation", {
+  x <- extract(object = A.VALUES, as.groups = TRUE, norm.per = "row")
+  expect_is(x, "data.frame")
+  expect_equal(dim(x), c(6L, 99L))
+  expect_true(is_ci_plottable(x))
+  got <- ci_plot(x[, 1L:9L])
+  expect_equal(got, c("1: Bacillus simplex 3", "2: Bacillus simplex 4"))
+})
+
+
+## extract
+test_that("one cannot pass too many 'Parameter' columns to group_ci()", {
+  Parameter <- rep("A", length(A.VALUES[, 1L]))
+  xy <- cbind(A.VALUES, Parameter)
+  expect_error(x <- extract(object = xy,
+    as.groups = colnames(xy[, c(1L:3L, 102L)]),
+    norm.per = "column", norm.by = TRUE))
+})
+
+
+## extract
+test_that("extract works with grouping and 'plate.div' normalisation", {
+  # 'as.groups' given as character-string of the column-names
+  x <- extract(object = A.VALUES, as.groups = colnames(A.VALUES[, 1L:2L]),
+    norm.per = "row", subtract = FALSE, norm.by = 10L)
+  expect_is(x, "data.frame")
+  expect_equal(dim(x), c(6L, 99L))
+  expect_true(is_ci_plottable(x))
+  got <- ci_plot(x[, 1L:6L])
+  expect_equal(got, c("1: Bacillus simplex 3", "2: Bacillus simplex 4"))
+})
+
+
+## extract
+test_that("extract works with grouping and 'well.div' normalisation", {
+  # as.groups given directly as character-string
+  x <- extract(object = A.VALUES, as.groups = c("organism", "run"),
+    norm.per = "column")
+  expect_is(x, "data.frame")
+  expect_equal(dim(x), c(6L, 99L))
+  expect_true(is_ci_plottable(x))
+  got <- ci_plot(x[, 1L:6L])
+  expect_equal(got, c("1: Bacillus simplex 3", "2: Bacillus simplex 4"))
+})
+
+
+## extract
+test_that("extract works with grouping and 'well.sub' normalisation", {
+  # only one column in as.groups
+  x <- extract(object = A.VALUES, as.groups = "organism",
+    norm.per = "row")
+  expect_is(x, "data.frame")
+  expect_equal(dim(x), c(3L, 98L))
+  expect_true(is_ci_plottable(x))
+  got <- ci_plot(x[, 1L:6L])
+  expect_equal(got, "1: Bacillus simplex")
+})
+
+
+## extract
+test_that("extract yields error if incorrect 'as.groups' is passed", {
+  # wrong columns in 'as.groups'-argument
+  expect_error(x <- extract(object = A.VALUES,
+    as.groups = c("organism", "run", "H06 (Acetoacetic Acid)"),
+    norm.per = "column"))
+  # => Error in extract(object = A.VALUES,
+  #   as.groups = c("Strain", "Experiment",  :
+  #   cannot find column name: H06 (Acetoacetic Acid)
+})
+
+## extract
+test_that("extract() deals with duplicate column names", {
+  x <- extract(object = A.VALUES,
+    as.groups = c("organism", "run", "organism"),
+    norm.per = "column", dup = "ignore")
+  # =>  Warning message:
+  # In extract(object = A.VALUES, as.groups = c("Strain", "Experiment",  :
+  #   as.groups variable(s) are not unique
+  expect_is(x, "data.frame")
+  expect_equal(dim(x), c(6L, 100L))
+  expect_true(is_ci_plottable(x))
+  got <- ci_plot(x[, 1L:10L])
+  expect_equal(got, c("1: Bacillus simplex 3 Bacillus simplex",
+    "2: Bacillus simplex 4 Bacillus simplex")) # duplication of names
+})
+
+
 ################################################################################
 
 
